#!/usr/bin/env python3
"""
Windows Display Manager (System Management Tool)

Parameters:
    --list: List all display devices
    --info DISPLAY: Show display information
    --resolution DISPLAY WxH: Set display resolution
    --refresh DISPLAY RATE: Set refresh rate
    --position DISPLAY X Y: Set display position
    --primary DISPLAY: Set primary display
    --format FORMAT: Output format (table/json)

Requires: win32api, wmi, tabulate packages
Warning: Invalid display settings may cause screen flickering
"""

import os
import sys
import json
import time
import ctypes
import logging
from typing import List, Dict, Optional, Tuple, Union
from dataclasses import dataclass
from enum import Enum, auto

import win32api
import win32con
import win32gui
from tabulate import tabulate

# Configure logging
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s'
)
logger = logging.getLogger(__name__)

class DisplayOrientation(Enum):
    """Display orientation options."""
    LANDSCAPE = 0
    PORTRAIT = 90
    LANDSCAPE_FLIPPED = 180
    PORTRAIT_FLIPPED = 270

@dataclass
class DisplayMode:
    """Display mode information."""
    width: int
    height: int
    frequency: int
    bits_per_pixel: int

@dataclass
class DisplayDevice:
    """Display device information."""
    name: str
    device_id: str
    is_primary: bool
    is_active: bool
    position: Tuple[int, int]
    resolution: Tuple[int, int]
    orientation: DisplayOrientation
    refresh_rate: int
    bits_per_pixel: int
    available_modes: List[DisplayMode]

class DisplayManager:
    """Manages Windows display settings."""
    
    def __init__(self):
        """Initialize the DisplayManager."""
        self.user32 = ctypes.windll.user32
    
    def get_display_devices(self) -> List[DisplayDevice]:
        """
        Get information about all display devices.
        
        Returns:
            List[DisplayDevice]: List of display devices
        """
        devices = []
        
        try:
            # Enumerate display devices
            i = 0
            while True:
                device_info = win32api.EnumDisplayDevices(None, i)
                if not device_info:
                    break
                    
                if device_info.StateFlags & win32con.DISPLAY_DEVICE_ATTACHED_TO_DESKTOP:
                    settings = win32api.EnumDisplaySettings(
                        device_info.DeviceName, win32con.ENUM_CURRENT_SETTINGS
                    )
                    
                    # Get available display modes
                    modes = []
                    mode_num = 0
                    while True:
                        try:
                            mode = win32api.EnumDisplaySettings(
                                device_info.DeviceName, mode_num
                            )
                            if not mode:
                                break
                            modes.append(DisplayMode(
                                width=mode.PelsWidth,
                                height=mode.PelsHeight,
                                frequency=mode.DisplayFrequency,
                                bits_per_pixel=mode.BitsPerPel
                            ))
                            mode_num += 1
                        except:
                            break
                    
                    # Create DisplayDevice object
                    device = DisplayDevice(
                        name=device_info.DeviceString,
                        device_id=device_info.DeviceName,
                        is_primary=bool(device_info.StateFlags & win32con.DISPLAY_DEVICE_PRIMARY_DEVICE),
                        is_active=bool(device_info.StateFlags & win32con.DISPLAY_DEVICE_ACTIVE),
                        position=(settings.Position_x, settings.Position_y),
                        resolution=(settings.PelsWidth, settings.PelsHeight),
                        orientation=DisplayOrientation(settings.DisplayOrientation * 90),
                        refresh_rate=settings.DisplayFrequency,
                        bits_per_pixel=settings.BitsPerPel,
                        available_modes=modes
                    )
                    devices.append(device)
                    
                i += 1
                
        except Exception as e:
            logger.error(f"Error getting display devices: {e}")
            
        return devices
    
    def set_display_resolution(self, device_id: str, width: int,
                             height: int, frequency: Optional[int] = None) -> bool:
        """
        Set display resolution for a device.
        
        Args:
            device_id: Display device ID
            width: Desired width in pixels
            height: Desired height in pixels
            frequency: Optional refresh rate in Hz
            
        Returns:
            bool: True if successful, False otherwise
        """
        try:
            # Get current settings
            settings = win32api.EnumDisplaySettings(
                device_id, win32con.ENUM_CURRENT_SETTINGS
            )
            
            # Update settings
            settings.PelsWidth = width
            settings.PelsHeight = height
            if frequency:
                settings.DisplayFrequency = frequency
            
            # Apply settings
            result = win32api.ChangeDisplaySettingsEx(
                device_id,
                settings,
                win32con.CDS_UPDATEREGISTRY
            )
            
            if result == win32con.DISP_CHANGE_SUCCESSFUL:
                logger.info(f"Successfully changed resolution to {width}x{height}")
                return True
            else:
                logger.error(f"Failed to change resolution: {result}")
                return False
                
        except Exception as e:
            logger.error(f"Error setting display resolution: {e}")
            return False
    
    def set_display_orientation(self, device_id: str,
                              orientation: DisplayOrientation) -> bool:
        """
        Set display orientation for a device.
        
        Args:
            device_id: Display device ID
            orientation: Desired orientation
            
        Returns:
            bool: True if successful, False otherwise
        """
        try:
            # Get current settings
            settings = win32api.EnumDisplaySettings(
                device_id, win32con.ENUM_CURRENT_SETTINGS
            )
            
            # Update settings
            settings.DisplayOrientation = orientation.value // 90
            
            # Apply settings
            result = win32api.ChangeDisplaySettingsEx(
                device_id,
                settings,
                win32con.CDS_UPDATEREGISTRY
            )
            
            if result == win32con.DISP_CHANGE_SUCCESSFUL:
                logger.info(f"Successfully changed orientation to {orientation.name}")
                return True
            else:
                logger.error(f"Failed to change orientation: {result}")
                return False
                
        except Exception as e:
            logger.error(f"Error setting display orientation: {e}")
            return False
    
    def set_display_position(self, device_id: str, x: int, y: int) -> bool:
        """
        Set display position for a device.
        
        Args:
            device_id: Display device ID
            x: X coordinate
            y: Y coordinate
            
        Returns:
            bool: True if successful, False otherwise
        """
        try:
            # Get current settings
            settings = win32api.EnumDisplaySettings(
                device_id, win32con.ENUM_CURRENT_SETTINGS
            )
            
            # Update settings
            settings.Position_x = x
            settings.Position_y = y
            
            # Apply settings
            result = win32api.ChangeDisplaySettingsEx(
                device_id,
                settings,
                win32con.CDS_UPDATEREGISTRY
            )
            
            if result == win32con.DISP_CHANGE_SUCCESSFUL:
                logger.info(f"Successfully changed position to ({x}, {y})")
                return True
            else:
                logger.error(f"Failed to change position: {result}")
                return False
                
        except Exception as e:
            logger.error(f"Error setting display position: {e}")
            return False
    
    def set_primary_display(self, device_id: str) -> bool:
        """
        Set primary display device.
        
        Args:
            device_id: Display device ID
            
        Returns:
            bool: True if successful, False otherwise
        """
        try:
            # Get current settings
            settings = win32api.EnumDisplaySettings(
                device_id, win32con.ENUM_CURRENT_SETTINGS
            )
            
            # Set primary flag
            settings.Position_x = 0
            settings.Position_y = 0
            settings.Flags |= win32con.CDS_SET_PRIMARY
            
            # Apply settings
            result = win32api.ChangeDisplaySettingsEx(
                device_id,
                settings,
                win32con.CDS_UPDATEREGISTRY | win32con.CDS_SET_PRIMARY
            )
            
            if result == win32con.DISP_CHANGE_SUCCESSFUL:
                logger.info(f"Successfully set {device_id} as primary display")
                return True
            else:
                logger.error(f"Failed to set primary display: {result}")
                return False
                
        except Exception as e:
            logger.error(f"Error setting primary display: {e}")
            return False
    
    def get_display_brightness(self) -> Optional[int]:
        """
        Get current display brightness.
        
        Returns:
            Optional[int]: Brightness level (0-100) or None if not supported
        """
        try:
            # This is a simplified implementation
            # In a real implementation, you would use WMI to get brightness
            return None
            
        except Exception as e:
            logger.error(f"Error getting display brightness: {e}")
            return None
    
    def set_display_brightness(self, level: int) -> bool:
        """
        Set display brightness.
        
        Args:
            level: Brightness level (0-100)
            
        Returns:
            bool: True if successful, False otherwise
        """
        try:
            # This is a simplified implementation
            # In a real implementation, you would use WMI to set brightness
            return False
            
        except Exception as e:
            logger.error(f"Error setting display brightness: {e}")
            return False

def format_display_info(info: Union[DisplayDevice, List[DisplayDevice]],
                       format_type: str = "table") -> str:
    """
    Format display information for output.
    
    Args:
        info: Display information to format
        format_type: Output format ("table" or "json")
        
    Returns:
        str: Formatted information
    """
    if format_type == "json":
        if isinstance(info, list):
            return json.dumps([vars(d) for d in info], indent=2)
        return json.dumps(vars(info), indent=2)
    
    if isinstance(info, list):
        if not info:
            return "No display devices found"
            
        headers = ["Name", "Resolution", "Refresh Rate", "Primary", "Active"]
        data = [
            [
                d.name,
                f"{d.resolution[0]}x{d.resolution[1]}",
                f"{d.refresh_rate} Hz",
                "Yes" if d.is_primary else "No",
                "Yes" if d.is_active else "No"
            ]
            for d in info
        ]
        
        return tabulate(data, headers=headers, tablefmt="grid")
        
    else:
        data = [
            ["Name", info.name],
            ["Device ID", info.device_id],
            ["Resolution", f"{info.resolution[0]}x{info.resolution[1]}"],
            ["Position", f"({info.position[0]}, {info.position[1]})"],
            ["Orientation", info.orientation.name],
            ["Refresh Rate", f"{info.refresh_rate} Hz"],
            ["Color Depth", f"{info.bits_per_pixel} bits"],
            ["Primary", "Yes" if info.is_primary else "No"],
            ["Active", "Yes" if info.is_active else "No"]
        ]
        
        return tabulate(data, headers=["Property", "Value"], tablefmt="grid")

def main():
    """Main function to handle command-line interface."""
    import argparse
    
    parser = argparse.ArgumentParser(description="Windows Display Manager")
    parser.add_argument("--list", action="store_true",
                      help="List display devices")
    parser.add_argument("--device",
                      help="Display device ID for operations")
    parser.add_argument("--resolution",
                      help="Set resolution (e.g., '1920x1080')")
    parser.add_argument("--refresh-rate", type=int,
                      help="Set refresh rate in Hz")
    parser.add_argument("--orientation",
                      choices=[o.name for o in DisplayOrientation],
                      help="Set display orientation")
    parser.add_argument("--position",
                      help="Set position (e.g., '0,0')")
    parser.add_argument("--set-primary", action="store_true",
                      help="Set as primary display")
    parser.add_argument("--brightness", type=int,
                      help="Set brightness level (0-100)")
    parser.add_argument("--format", choices=["table", "json"],
                      default="table",
                      help="Output format (table or json)")
    
    args = parser.parse_args()
    
    display_manager = DisplayManager()
    
    try:
        if args.list:
            devices = display_manager.get_display_devices()
            print(format_display_info(devices, args.format))
            
        elif args.device:
            devices = display_manager.get_display_devices()
            device = next((d for d in devices if d.device_id == args.device), None)
            
            if not device:
                print(f"Display device not found: {args.device}")
                sys.exit(1)
            
            if args.resolution:
                try:
                    width, height = map(int, args.resolution.split("x"))
                    if display_manager.set_display_resolution(
                        args.device, width, height, args.refresh_rate
                    ):
                        print("Resolution changed successfully")
                    else:
                        print("Failed to change resolution")
                        sys.exit(1)
                except ValueError:
                    print("Invalid resolution format. Use 'WIDTHxHEIGHT'")
                    sys.exit(1)
                    
            elif args.orientation:
                if display_manager.set_display_orientation(
                    args.device, DisplayOrientation[args.orientation]
                ):
                    print("Orientation changed successfully")
                else:
                    print("Failed to change orientation")
                    sys.exit(1)
                    
            elif args.position:
                try:
                    x, y = map(int, args.position.split(","))
                    if display_manager.set_display_position(args.device, x, y):
                        print("Position changed successfully")
                    else:
                        print("Failed to change position")
                        sys.exit(1)
                except ValueError:
                    print("Invalid position format. Use 'X,Y'")
                    sys.exit(1)
                    
            elif args.set_primary:
                if display_manager.set_primary_display(args.device):
                    print("Primary display set successfully")
                else:
                    print("Failed to set primary display")
                    sys.exit(1)
                    
            elif args.brightness is not None:
                if 0 <= args.brightness <= 100:
                    if display_manager.set_display_brightness(args.brightness):
                        print("Brightness changed successfully")
                    else:
                        print("Failed to change brightness")
                        sys.exit(1)
                else:
                    print("Brightness must be between 0 and 100")
                    sys.exit(1)
                    
            else:
                print(format_display_info(device, args.format))
                
        else:
            parser.print_help()
            
    except Exception as e:
        logger.error(f"An error occurred: {e}")
        sys.exit(1)

if __name__ == "__main__":
    main()
